#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <signal.h>

#include <cra_debug.h>

const char *progname;

void cdb_backtrace(const char *const filename)
{
  char ts[65536];
  int pid;

  pid = fork();
  if (pid == -1) {
    perror("fork");
  }
  if (pid == 0) {
    sleep(10);
    exit(0);
  } else {
    /* Parent */
    snprintf(ts, sizeof(ts)
	     , "echo -e 'set verbose off\nfile %s\n"
	     "attach %d\nbt' | gdb -n -batch -x - "
	     "| sed -e '1,/^#[0-9].*backtrace/d' -e '/(gdb) /,$d'"
	     "> %s"
	     , progname, pid, filename);

    /* If this fails, error messages will do to stdout, no additional
     * checking here.
     */
    system(ts);

    /* Just in case. No error checking */
    kill(pid, SIGKILL);

    waitpid(pid, NULL, 0);
  }
}

void cdb_setprogname(const char *const s)
{
  progname = strdup(s);
  if (progname == NULL) {
    perror("cra_debug: strdup failed");
  }
}

#ifdef TEST
static void sub2(int blubb)
{
  cdb_backtrace("bt");
  printf("Survived: %d\n", blubb);
}

static void sub1(int bla)
{
  sub2(bla++);
}

int main(int argc, char *argv[])
{
  cdb_setprogname(argv[0]);
  sub1(5);
  return 0;
}
#endif
